@@ -34,45 +34,51 @@
 namespace Twitterizer
 {
     using System;
+    using System.Linq;
     using System.Diagnostics;
     using Newtonsoft.Json;
     using Twitterizer.Core;
+    using Twitterizer.Entities;
+    using System.Runtime.Serialization;
 
     /// <include file='TwitterStatus.xml' path='TwitterStatus/TwitterStatus/*'/>
     [JsonObject(MemberSerialization.OptIn)]
+#if !SILVERLIGHT
     [Serializable]
+#endif
     [DebuggerDisplay("{User.ScreenName}/{Text}")]
+    [DataContract]
     public class TwitterStatus : TwitterObject
     {
-        /// <summary>
-        /// Initializes a new instance of the <see cref="TwitterStatus"/> class.
-        /// </summary>
-        public TwitterStatus()
-            : base()
-        {
-        }
-
         #region Properties
         /// <summary>
         /// Gets or sets the status id.
         /// </summary>
         /// <value>The status id.</value>
-        [JsonProperty(PropertyName = "id")]
+        [DataMember, JsonProperty(PropertyName = "id")]
         public decimal Id { get; set; }
 
+        /// <summary>
+        /// Gets or sets the string id.
+        /// </summary>
+        /// <value>The string id.</value>
+        [DataMember, JsonProperty(PropertyName = "id_str")]
+        public string StringId { get; set; }
+
         /// <summary>
         /// Gets or sets a value indicating whether this status message is truncated.
         /// </summary>
         /// <value>
         /// <c>true</c> if this status message is truncated; otherwise, <c>false</c>.
         /// </value>
-        [JsonProperty(PropertyName = "truncated")]
+        [DataMember, JsonProperty(PropertyName = "truncated")]
         public bool? IsTruncated { get; set; }
 
         /// <summary>
         /// Gets or sets the created date.
         /// </summary>
         /// <value>The created date.</value>
+        [DataMember]
         [JsonProperty(PropertyName = "created_at")]
         [JsonConverter(typeof(TwitterizerDateConverter))]
         public DateTime CreatedDate { get; set; }
@@ -81,28 +87,28 @@ namespace Twitterizer
         /// Gets or sets the source.
         /// </summary>
         /// <value>The source.</value>
-        [JsonProperty(PropertyName = "source")]
+        [DataMember, JsonProperty(PropertyName = "source")]
         public string Source { get; set; }
 
         /// <summary>
         /// Gets or sets the screenName the status is in reply to.
         /// </summary>
         /// <value>The screenName.</value>
-        [JsonProperty(PropertyName = "in_reply_to_screen_name")]
+        [DataMember, JsonProperty(PropertyName = "in_reply_to_screen_name")]
         public string InReplyToScreenName { get; set; }
 
         /// <summary>
         /// Gets or sets the user id the status is in reply to.
         /// </summary>
         /// <value>The user id.</value>
-        [JsonProperty(PropertyName = "in_reply_to_user_id")]
+        [DataMember, JsonProperty(PropertyName = "in_reply_to_user_id")]
         public decimal? InReplyToUserId { get; set; }
 
         /// <summary>
         /// Gets or sets the status id the status is in reply to.
         /// </summary>
         /// <value>The status id.</value>
-        [JsonProperty(PropertyName = "in_reply_to_status_id")]
+        [DataMember, JsonProperty(PropertyName = "in_reply_to_status_id")]
         public decimal? InReplyToStatusId { get; set; }
 
         /// <summary>
@@ -111,80 +117,183 @@ namespace Twitterizer
         /// <value>
         /// <c>true</c> if this instance is favorited; otherwise, <c>false</c>.
         /// </value>
-        [JsonProperty(PropertyName = "favorited")]
+        [DataMember, JsonProperty(PropertyName = "favorited")]
         public bool? IsFavorited { get; set; }
 
         /// <summary>
         /// Gets or sets the text of the status.
         /// </summary>
         /// <value>The status text.</value>
-        [JsonProperty(PropertyName = "text")]
+        [DataMember, JsonProperty(PropertyName = "text")]
         public string Text { get; set; }
 
         /// <summary>
         /// Gets or sets the user.
         /// </summary>
         /// <value>The user that posted this status.</value>
-        [JsonProperty(PropertyName = "user")]
+        [DataMember, JsonProperty(PropertyName = "user")]
         public TwitterUser User { get; set; }
 
         /// <summary>
         /// Gets or sets the retweeted status.
         /// </summary>
         /// <value>The retweeted status.</value>
-        [JsonProperty(PropertyName = "retweeted_status")]
+        [DataMember, JsonProperty(PropertyName = "retweeted_status")]
         public TwitterStatus RetweetedStatus { get; set; }
 
         /// <summary>
         /// Gets or sets the place.
         /// </summary>
         /// <value>The place.</value>
-        [JsonProperty(PropertyName = "place")]
+        [DataMember, JsonProperty(PropertyName = "place")]
         public TwitterPlace Place { get; set; }
 
         /// <summary>
         /// Gets or sets the geo location data.
         /// </summary>
         /// <value>The geo location data.</value>
-        [JsonProperty(PropertyName = "geo")]
+        [DataMember, JsonProperty(PropertyName = "geo")]
         public TwitterGeo Geo { get; set; }
 
         /// <summary>
         /// Gets or sets the entities.
         /// </summary>
         /// <value>The entities.</value>
-        [JsonProperty(PropertyName  = "entities")]
+        [DataMember]
+        [JsonProperty(PropertyName = "entities")]
         [JsonConverter(typeof(Entities.TwitterEntityCollection.Converter))]
         public Entities.TwitterEntityCollection Entities { get; set; }
 
         /// <summary>
-        /// Gets or sets the retweet count.
+        /// Gets or sets the retweet count string.
         /// </summary>
         /// <value>The retweet count.</value>
-        [JsonProperty(PropertyName = "retweet_count")]
-        public int? RetweetCount { get; set; }
+        [DataMember, JsonProperty(PropertyName = "retweet_count")]
+        public string RetweetCountString { get; set; }
+
+        /// <summary>
+        /// Gets the retweet count.
+        /// </summary>
+        /// <value>The retweet count.</value>
+        [DataMember]
+        public int? RetweetCount
+        {
+            get
+            {
+                if (string.IsNullOrEmpty(this.RetweetCountString)) return null;
+
+                int parsedResult = 0;
+
+                if (
+                    this.RetweetCountString.EndsWith("+") &&
+                    !int.TryParse(this.RetweetCountString.Substring(0, this.RetweetCountString.Length - 1), out parsedResult)
+                    )
+                {
+                    return null;
+                }
+
+                if (!int.TryParse(this.RetweetCountString, out parsedResult))
+                {
+                    return null;
+                }
+
+                return parsedResult;
+            }
+        }
+
+        /// <summary>
+        /// Gets a value indicating that the number of retweets exceeds the reported value in RetweetCount. For example, "more than 100"
+        /// </summary>
+        /// <value>The retweet count plus indicator.</value>
+        [DataMember]
+        public bool? RetweetCountPlus
+        {
+            get
+            {
+                if (string.IsNullOrEmpty(this.RetweetCountString)) return null;
+
+                return this.RetweetCountString.EndsWith("+");
+            }
+        }
 
         /// <summary>
         /// Gets or sets a value indicating whether this <see cref="TwitterStatus"/> is retweeted.
         /// </summary>
         /// <value><c>true</c> if retweeted; otherwise, <c>false</c>.</value>
-        [JsonProperty(PropertyName = "retweeted")]
+        [DataMember, JsonProperty(PropertyName = "retweeted")]
         public bool Retweeted { get; set; }
         #endregion
 
         /// <summary>
-        /// Updates the authenticated user's status to the supplied text.
+        /// Returns the status text with HTML links to users, urls, and hashtags.
+        /// </summary>
+        /// <returns></returns>
+        public string LinkifiedText()
+        {
+            if (this.Entities == null || this.Entities.Count == 0)
+            {
+                return this.Text;
+            }
+
+            string linkedText = this.Text;
+
+            var entitiesSorted = this
+                                  .Entities
+                                  .OrderBy(e => e.StartIndex)
+                                  .Reverse();
+
+            foreach (TwitterEntity entity in entitiesSorted)
+            {
+                if (entity is TwitterHashTagEntity)
+                {
+                    TwitterHashTagEntity tagEntity = (TwitterHashTagEntity)entity;
+
+                    linkedText = string.Format(
+                        "{0}<a href=\"http://twitter.com/search?q=%23{1}\">{1}</a>{2}",
+                        linkedText.Substring(0, entity.StartIndex),
+                        tagEntity.Text,
+                        linkedText.Substring(entity.EndIndex));
+                }
+
+                if (entity is TwitterUrlEntity)
+                {
+                    TwitterUrlEntity urlEntity = (TwitterUrlEntity)entity;
+
+                    linkedText = string.Format(
+                        "{0}<a href=\"{1}\">{1}</a>{2}",
+                        linkedText.Substring(0, entity.StartIndex),
+                        urlEntity.Url,
+                        linkedText.Substring(entity.EndIndex));
+                }
+
+                if (entity is TwitterMentionEntity)
+                {
+                    TwitterMentionEntity mentionEntity = (TwitterMentionEntity)entity;
+
+                    linkedText = string.Format(
+                        "{0}<a href=\"http://twitter.com/{1}\">@{1}</a>{2}",
+                        linkedText.Substring(0, entity.StartIndex),
+                        mentionEntity.ScreenName,
+                        linkedText.Substring(entity.EndIndex));
+                }
+            }
+
+            return linkedText;
+        }
+
+        /// <summary>
+        /// Updates the authenticating user's status. A status update with text identical to the authenticating user's text identical to the authenticating user's current status will be ignored to prevent duplicates.
         /// </summary>
         /// <param name="tokens">The tokens.</param>
         /// <param name="text">The status text.</param>
         /// <returns>A <see cref="TwitterStatus"/> object of the newly created status.</returns>
-        public static TwitterStatus Update(OAuthTokens tokens, string text)
+        public static TwitterResponse<TwitterStatus> Update(OAuthTokens tokens, string text)
         {
             return Update(tokens, text, null);
         }
 
         /// <summary>
-        /// Updates the specified tokens.
+        /// Updates the authenticating user's status. A status update with text identical to the authenticating user's text identical to the authenticating user's current status will be ignored to prevent duplicates.
         /// </summary>
         /// <param name="tokens">The tokens.</param>
         /// <param name="text">The status text.</param>
@@ -192,9 +301,39 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatus"/> object of the newly created status.
         /// </returns>
-        public static TwitterStatus Update(OAuthTokens tokens, string text, StatusUpdateOptions options)
+        public static TwitterResponse<TwitterStatus> Update(OAuthTokens tokens, string text, StatusUpdateOptions options)
         {
-            return CommandPerformer<TwitterStatus>.PerformAction(new Commands.UpdateStatusCommand(tokens, text, options));
+            return CommandPerformer.PerformAction(new Commands.UpdateStatusCommand(tokens, text, options));
+        }
+
+		/// <summary>
+		/// Updates the authenticating user's status. A status update with text identical to the authenticating user's text identical to the authenticating user's current status will be ignored to prevent duplicates.
+		/// </summary>
+		/// <param name="tokens">The tokens.</param>
+		/// <param name="text">The status text.</param>
+		/// <param name="file">The file to upload.</param>
+		/// <param name="options">The options.</param>
+		/// <returns>
+		/// A <see cref="TwitterStatus"/> object of the newly created status.
+		/// </returns>
+		public static TwitterResponse<TwitterStatus> UpdateWithMedia(OAuthTokens tokens, string text, byte[] fileData, StatusUpdateOptions options = null)
+		{
+            return CommandPerformer.PerformAction(new Commands.UpdateWithMediaCommand(tokens, text, fileData, options));
+		}
+
+        /// <summary>
+        /// Updates the authenticating user's status. A status update with text identical to the authenticating user's text identical to the authenticating user's current status will be ignored to prevent duplicates.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="text">The status text.</param>
+        /// <param name="fileLocation">The file location.</param>
+        /// <param name="options">The options.</param>
+        /// <returns>
+        /// A <see cref="TwitterStatus"/> object of the newly created status.
+        /// </returns>
+        public static TwitterResponse<TwitterStatus> UpdateWithMedia(OAuthTokens tokens, string text, string fileLocation, StatusUpdateOptions options = null)
+        {
+            return UpdateWithMedia(tokens, text, System.IO.File.ReadAllBytes(fileLocation), options);
         }
 
         /// <summary>
@@ -206,11 +345,11 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatus"/> object of the deleted status.
         /// </returns>
-        public static TwitterStatus Delete(OAuthTokens tokens, decimal id, OptionalProperties options)
+        public static TwitterResponse<TwitterStatus> Delete(OAuthTokens tokens, decimal id, OptionalProperties options)
         {
             Commands.DeleteStatusCommand command = new Twitterizer.Commands.DeleteStatusCommand(tokens, id, options);
 
-            return CommandPerformer<TwitterStatus>.PerformAction(command);
+            return CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -219,7 +358,7 @@ namespace Twitterizer
         /// <param name="tokens">The oauth tokens.</param>
         /// <param name="id">The status id.</param>
         /// <returns>A <see cref="TwitterStatus"/> object of the deleted status.</returns>
-        public static TwitterStatus Delete(OAuthTokens tokens, decimal id)
+        public static TwitterResponse<TwitterStatus> Delete(OAuthTokens tokens, decimal id)
         {
             return Delete(tokens, id, null);
         }
@@ -231,9 +370,9 @@ namespace Twitterizer
         /// <param name="statusId">The status id.</param>
         /// <param name="options">The options.</param>
         /// <returns>A <see cref="TwitterStatus"/> instance.</returns>
-        public static TwitterStatus Show(OAuthTokens tokens, decimal statusId, OptionalProperties options)
+        public static TwitterResponse<TwitterStatus> Show(OAuthTokens tokens, decimal statusId, OptionalProperties options)
         {
-            return CommandPerformer<TwitterStatus>.PerformAction(new Commands.ShowStatusCommand(tokens, statusId, options));
+            return CommandPerformer.PerformAction(new Commands.ShowStatusCommand(tokens, statusId, options));
         }
 
         /// <summary>
@@ -242,7 +381,7 @@ namespace Twitterizer
         /// <param name="tokens">The tokens.</param>
         /// <param name="statusId">The status id.</param>
         /// <returns>A <see cref="TwitterStatus"/> instance.</returns>
-        public static TwitterStatus Show(OAuthTokens tokens, decimal statusId)
+        public static TwitterResponse<TwitterStatus> Show(OAuthTokens tokens, decimal statusId)
         {
             return Show(tokens, statusId, null);
         }
@@ -252,7 +391,7 @@ namespace Twitterizer
         /// </summary>
         /// <param name="statusId">The status id.</param>
         /// <returns>A <see cref="TwitterStatus"/> instance.</returns>
-        public static TwitterStatus Show(decimal statusId)
+        public static TwitterResponse<TwitterStatus> Show(decimal statusId)
         {
             return Show(null, statusId);
         }
@@ -264,9 +403,9 @@ namespace Twitterizer
         /// <param name="statusId">The status id.</param>
         /// <param name="options">The options.</param>
         /// <returns>A <see cref="TwitterStatus"/> representing the newly created tweet.</returns>
-        public static TwitterStatus Retweet(OAuthTokens tokens, decimal statusId, OptionalProperties options)
+        public static TwitterResponse<TwitterStatus> Retweet(OAuthTokens tokens, decimal statusId, OptionalProperties options)
         {
-            return CommandPerformer<TwitterStatus>.PerformAction(
+            return CommandPerformer.PerformAction(
                 new Commands.RetweetCommand(tokens, statusId, options));
         }
 
@@ -276,7 +415,7 @@ namespace Twitterizer
         /// <param name="tokens">The tokens.</param>
         /// <param name="statusId">The status id.</param>
         /// <returns>A <see cref="TwitterStatus"/> representing the newly created tweet.</returns>
-        public static TwitterStatus Retweet(OAuthTokens tokens, decimal statusId)
+        public static TwitterResponse<TwitterStatus> Retweet(OAuthTokens tokens, decimal statusId)
         {
             return Retweet(tokens, statusId, null);
         }
@@ -290,9 +429,9 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatusCollection"/> instance.
         /// </returns>
-        public static TwitterStatusCollection Retweets(OAuthTokens tokens, decimal statusId, RetweetsOptions options)
+        public static TwitterResponse<TwitterStatusCollection> Retweets(OAuthTokens tokens, decimal statusId, RetweetsOptions options)
         {
-            return CommandPerformer<TwitterStatusCollection>.PerformAction(
+            return CommandPerformer.PerformAction(
                 new Commands.RetweetsCommand(tokens, statusId, options));
         }
 
@@ -302,7 +441,7 @@ namespace Twitterizer
         /// <param name="tokens">The tokens.</param>
         /// <param name="statusId">The status id.</param>
         /// <returns>A <see cref="TwitterStatusCollection"/> instance.</returns>
-        public static TwitterStatusCollection Retweets(OAuthTokens tokens, decimal statusId)
+        public static TwitterResponse<TwitterStatusCollection> Retweets(OAuthTokens tokens, decimal statusId)
         {
             return Retweets(tokens, statusId, null);
         }
@@ -315,7 +454,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatus"/> representing the newly created tweet.
         /// </returns>
-        public TwitterStatus Retweet(OAuthTokens tokens, OptionalProperties options)
+        public TwitterResponse<TwitterStatus> Retweet(OAuthTokens tokens, OptionalProperties options)
         {
             return Retweet(tokens, this.Id, options);
         }
@@ -327,7 +466,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatus"/> representing the newly created tweet.
         /// </returns>
-        public TwitterStatus Retweet(OAuthTokens tokens)
+        public TwitterResponse<TwitterStatus> Retweet(OAuthTokens tokens)
         {
             return Retweet(tokens, this.Id, null);
         }
@@ -340,7 +479,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatus"/> object of the deleted status.
         /// </returns>
-        public TwitterStatus Delete(OAuthTokens tokens, OptionalProperties options)
+        public TwitterResponse<TwitterStatus> Delete(OAuthTokens tokens, OptionalProperties options)
         {
             return Delete(tokens, this.Id, options);
         }
@@ -352,9 +491,36 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatus"/> object of the deleted status.
         /// </returns>
-        public TwitterStatus Delete(OAuthTokens tokens)
+        public TwitterResponse<TwitterStatus> Delete(OAuthTokens tokens)
         {
             return Delete(tokens, this.Id, null);
         }
+
+        /// <summary>
+        /// Shows Related Results of a tweet. Requires the id parameter of the tweet you are getting results for.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="statusId">The status id.</param>
+        /// <returns>A <see cref="TwitterStatus"/> representing the newly created tweet.</returns>
+        /// <remarks></remarks>
+        public static TwitterResponse<TwitterRelatedTweetsCollection> RelatedResultsShow(OAuthTokens tokens, decimal statusId)
+        {
+            return CommandPerformer.PerformAction(
+                new Commands.RelatedResultsCommand(tokens, statusId, null));
+        }
+
+
+        /// <summary>
+        /// Shows Related Results of a tweet. Requires the id parameter of the tweet you are getting results for.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="statusId">The status id.</param>
+        /// <param name="options">The options.</param>
+        /// <returns>A <see cref="TwitterStatus"/> representing the newly created tweet.</returns>
+        public static TwitterResponse<TwitterRelatedTweetsCollection> RelatedResultsShow(OAuthTokens tokens, decimal statusId, OptionalProperties options)
+        {
+            return CommandPerformer.PerformAction(
+                new Commands.RelatedResultsCommand(tokens, statusId, options));
+        }
     }
 }
