@@ -78,6 +78,9 @@ namespace Smuxi.Engine
         bool                    f_Listening;
         bool                    f_IsConnected;
 
+        int                     ErrorResponseCount { get; set; }
+        const int               MaxErrorResponseCount = 3;
+
         public override string NetworkID {
             get {
                 if (f_TwitterUser == null) {
@@ -169,14 +172,16 @@ namespace Smuxi.Engine
             var proxySettings = new ProxySettings();
             proxySettings.ApplyConfig(Session.UserConfig);
             var twitterUrl = new OptionalProperties().APIBaseAddress;
-            f_WebProxy = proxySettings.GetWebProxy(twitterUrl);
+            var proxy = proxySettings.GetWebProxy(twitterUrl);
             // HACK: Twitterizer will always use the system proxy if set to null
             // so explicitely override this by setting an empty proxy
-            if (f_WebProxy == null) {
+            if (proxy == null) {
                 f_WebProxy = new WebProxy();
+            } else {
+                f_WebProxy = proxy;
             }
-            f_OptionalProperties = CreateOptions<OptionalProperties>();
 
+            f_OptionalProperties = CreateOptions<OptionalProperties>();
             f_ProtocolChat = new ProtocolChatModel(NetworkID, "Twitter " + f_Username, this);
             f_ProtocolChat.InitMessageBuffer(
                 MessageBufferPersistencyType.Volatile
@@ -185,6 +190,16 @@ namespace Smuxi.Engine
             Session.AddChat(f_ProtocolChat);
             Session.SyncChat(f_ProtocolChat);
 
+            MessageBuilder builder;
+            if (proxy != null && proxy.Address != null) {
+                builder = CreateMessageBuilder();
+                builder.AppendEventPrefix();
+                builder.AppendText(_("Using proxy: {0}:{1}"),
+                                   proxy.Address.Host,
+                                   proxy.Address.Port);
+                Session.AddMessageToChat(Chat, builder.ToMessage());
+            }
+
             string msg;
             msg = String.Format(_("Connecting to Twitter..."));
             fm.SetStatus(msg);
@@ -195,7 +210,6 @@ namespace Smuxi.Engine
                 f_OAuthTokens.ConsumerKey = key[0];
                 f_OAuthTokens.ConsumerSecret = key[1];
 
-                MessageBuilder builder;
                 var password = server.Password ?? String.Empty;
                 var access = password.Split('|');
                 if (access.Length == 2) {
@@ -1453,6 +1467,7 @@ namespace Smuxi.Engine
             switch (response.Result) {
                 case RequestResult.Success:
                     // no error at all
+                    ErrorResponseCount = 0;
                     return false;
                 case RequestResult.ConnectionFailure:
                 case RequestResult.RateLimited:
@@ -1470,12 +1485,33 @@ namespace Smuxi.Engine
                     return true;
             }
 
+            if (ErrorResponseCount++ < MaxErrorResponseCount) {
 #if LOG4NET
-            f_Logger.Debug("IsTemporilyErrorResponse(): " +
-                           "Detected permanent error " +
-                           "RequestUrl: " + response.RequestUrl + " " +
-                           "Result: " + response.Result + " " +
-                           "Content:\n" + response.Content);
+                f_Logger.WarnFormat(
+                    "IsTemporilyErrorResponse(): Ignoring permanent error " +
+                    "({0}/{1}) " +
+                    "RequestUrl: {2} " +
+                    "Result: {3} " +
+                    "Content:\n{4}",
+                    ErrorResponseCount,
+                    MaxErrorResponseCount,
+                    response.RequestUrl,
+                    response.Result,
+                    response.Content
+                );
+#endif
+                return true;
+            }
+
+#if LOG4NET
+            f_Logger.ErrorFormat(
+                "IsTemporilyErrorResponse(): Detected permanent error " +
+                "RequestUrl: {0} Result: {1} " +
+                "Content:\n{2}",
+                response.RequestUrl,
+                response.Result,
+                response.Content
+            );
 #endif
             return false;
         }
