@@ -35,7 +35,8 @@ namespace Smuxi.Engine
         private DateTime                f_TimeStamp;
         private IList<MessagePartModel> f_MessageParts;
         private MessageType             f_MessageType;
-        public  bool                    IsCompactable { get; set; }
+        [NonSerialized]
+        private bool                    f_IsCompactable;
 
         public DateTime TimeStamp {
             get {
@@ -51,6 +52,12 @@ namespace Smuxi.Engine
                 return f_MessageParts;
             }
         }
+
+        public bool IsEmpty {
+            get {
+                return f_MessageParts.Count == 0;
+            }
+        }
         
         public MessageType MessageType {
             get {
@@ -61,6 +68,15 @@ namespace Smuxi.Engine
             }
         }
         
+        public bool IsCompactable {
+            get {
+                return f_IsCompactable;
+            }
+            set {
+                f_IsCompactable = value;
+            }
+        }
+
         public MessageModel()
         {
             f_TimeStamp    = DateTime.UtcNow;
@@ -78,6 +94,19 @@ namespace Smuxi.Engine
         {
         }
         
+        public MessageModel(MessageModel msg) : this()
+        {
+            if (msg == null) {
+                throw new ArgumentNullException("msg");
+            }
+
+            var writer = SerializationWriter.GetWriter();
+            msg.GetObjectData(writer);
+            var data = writer.GetData();
+            var reader = SerializationReader.GetReader(data);
+            SetObjectData(reader);
+        }
+
         protected MessageModel(SerializationInfo info, StreamingContext ctx)
         {
             SerializationReader sr = SerializationReader.GetReader(info);
@@ -122,6 +151,11 @@ namespace Smuxi.Engine
             return sb.ToString();
         }
 
+        public override int GetHashCode()
+        {
+            return f_TimeStamp.GetHashCode();
+        }
+
         public override bool Equals(object obj)
         {
             if (!(obj is MessageModel)) {
