@@ -419,6 +419,7 @@ namespace Smuxi.Engine
             string[] help = {
                 "help",
                 "connect/server protocol [protocol-parameters]",
+                "connect/server network",
                 "disconnect [server]",
                 "network list",
                 "network close [network]",
@@ -444,9 +445,15 @@ namespace Smuxi.Engine
             }
             
             FrontendManager fm = cd.FrontendManager;
-            
-            string protocol;
-            if (cd.DataArray.Length >= 3) {
+
+            string protocol = null;
+            ServerModel server = null;
+            // first lookup by network name
+            if (cd.DataArray.Length == 2) {
+                var network = cd.Parameter;
+                var serverSettings = new ServerListController(UserConfig);
+                server = serverSettings.GetServerByNetwork(network);
+            } else if (cd.DataArray.Length >= 3) {
                 protocol = cd.DataArray[1];
             } else if (cd.DataArray.Length >= 2) {
                 // HACK: simply assume the user meant irc if not specified as
@@ -479,7 +486,7 @@ namespace Smuxi.Engine
             }
             */
 
-            if (protocolManager == null) {
+            if (protocolManager == null && server == null) {
                 try {
                     protocolManager = CreateProtocolManager(protocol);
                     _ProtocolManagers.Add(protocolManager);
@@ -510,7 +517,11 @@ namespace Smuxi.Engine
             // run in background so it can't block the command queue
             ThreadPool.QueueUserWorkItem(delegate {
                 try {
-                    protocolManager.Command(cd);
+                    if (protocolManager == null && server != null) {
+                        protocolManager = Connect(server, fm);
+                    } else {
+                        protocolManager.Command(cd);
+                    }
 
                     // set this as current protocol manager
                     // but only if there was none set (we might be on a chat for example)
@@ -658,7 +669,13 @@ namespace Smuxi.Engine
             if (cd == null) {
                 throw new ArgumentNullException("cd");
             }
-            
+
+            if (cd.DataArray.Length == 1) {
+                // no parameter given, fallback to list
+                _CommandNetworkList(cd);
+                return;
+            }
+
             FrontendManager fm = cd.FrontendManager;
             if (cd.DataArray.Length >= 2) {
                 switch (cd.DataArray[1].ToLower()) {
@@ -681,19 +698,64 @@ namespace Smuxi.Engine
             }
         }
         
-        private void _CommandNetworkList(CommandModel cd)
+        private void _CommandNetworkList(CommandModel cmd)
         {
-            FrontendManager fm = cd.FrontendManager;
-            fm.AddTextToChat(cd.Chat, "-!- " + _("Networks") + ":");
+            var frontend = cmd.FrontendManager;
+            var servers = new ServerListController(UserConfig);
+            var availableNetworks = servers.GetNetworks();
+            var connectedNetworks = new List<IProtocolManager>();
             lock (_ProtocolManagers) {
-                foreach (IProtocolManager nm in _ProtocolManagers) {
-                    fm.AddTextToChat(cd.Chat, "-!- " +
-                        _("Protocol") + ": " + nm.Protocol + " " +
-                        _("Network") + ": " + nm.NetworkID + " " +
-                        _("Host") + ": " + nm.Host + " " +
-                        _("Port") + ": " + nm.Port);
+                foreach (IProtocolManager pm in _ProtocolManagers) {
+                    if (pm.IsConnected) {
+                        connectedNetworks.Add(pm);
+                        if (!String.IsNullOrEmpty(pm.NetworkID)) {
+                            availableNetworks.Remove(pm.NetworkID);
+                        }
+                    }
                 }
             }
+
+            var msg = CreateMessageBuilder();
+            // TRANSLATOR: this line is used as a label / category for a
+            // list of networks below
+            msg.AppendHeader(_("Connected Networks"));
+            frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+            foreach (var network in connectedNetworks) {
+                msg = CreateMessageBuilder();
+                msg.AppendEventPrefix();
+                msg.AppendText("{0}: {1} ", _("Network"),  network.NetworkID);
+                msg.AppendText("{0}: {1} ", _("Protocol"), network.Protocol);
+                msg.AppendText("{0}: {1} ", _("Host"),     network.Host);
+                msg.AppendText("{0}: {1}",  _("Port"),     network.Port);
+                frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+            }
+            if (connectedNetworks.Count == 0) {
+                msg = CreateMessageBuilder();
+                // TRANSLATOR: no connected networks
+                msg.AppendEventPrefix().AppendText("<{0}>", _("None"));
+                frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+            }
+
+            msg = CreateMessageBuilder();
+            // TRANSLATOR: this line is used as a label / category for a
+            // list of networks below
+            msg.AppendHeader(_("Available Networks"));
+            frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+            foreach (var network in availableNetworks) {
+                if (network == null || network.Trim().Length == 0) {
+                    continue;
+                }
+                msg = CreateMessageBuilder();
+                msg.AppendEventPrefix();
+                msg.AppendText("{0}: {1}", _("Network"), network);
+                frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+            }
+            if (availableNetworks.Count == 0) {
+                msg = CreateMessageBuilder();
+                // TRANSLATOR: no available networks
+                msg.AppendEventPrefix().AppendText("<{0}>", _("None"));
+                frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+            }
         }
         
         private void _CommandNetworkClose(CommandModel cd)
