@@ -1,13 +1,7 @@
 /*
- * $Id: GroupChatView.cs 188 2007-04-21 22:03:54Z meebey $
- * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/GroupChatView.cs $
- * $Rev: 188 $
- * $Author: meebey $
- * $Date: 2007-04-22 00:03:54 +0200 (Sun, 22 Apr 2007) $
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2007 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -27,6 +21,7 @@
  */
 
 using System;
+using System.Threading;
 using System.Collections.Generic;
 using System.Globalization;
 using Smuxi.Engine;
@@ -47,8 +42,6 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call(groupChat);
 
-            _IrcProtocolManager = (IrcProtocolManager) groupChat.ProtocolManager;
-            
             if (PersonTreeView != null) {
                 Gtk.CellRendererText cellr = new Gtk.CellRendererText();
                 // HACK: for some reason GTK is giving the space of 2 chars which
@@ -65,6 +58,15 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        public override void Sync()
+        {
+            Trace.Call();
+
+            base.Sync();
+
+            _IrcProtocolManager = (IrcProtocolManager) ProtocolManager;
+        }
+
         private void _RenderIrcGroupPersonMode(Gtk.TreeViewColumn column,
                                                Gtk.CellRenderer cellr,
                                                Gtk.TreeModel model, Gtk.TreeIter iter)
@@ -102,14 +104,20 @@ namespace Smuxi.Frontend.Gnome
             foreach (PersonModel person in persons) {
                 nicks.Add(person.ID);
             }
-            _IrcProtocolManager.CommandOp(
-                new CommandModel(
-                    Frontend.FrontendManager,
-                    ChatModel,
-                    String.Join(" ", nicks.ToArray())
-                )
-            );
-        } 
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    _IrcProtocolManager.CommandOp(
+                        new CommandModel(
+                            Frontend.FrontendManager,
+                            ChatModel,
+                            String.Join(" ", nicks.ToArray())
+                        )
+                    );
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                }
+            });
+        }
         
         private void _OnUserListMenuDeopActivated(object sender, EventArgs e)
         {
@@ -125,13 +133,19 @@ namespace Smuxi.Frontend.Gnome
             foreach (PersonModel person in persons) {
                 nicks.Add(person.ID);
             }
-            _IrcProtocolManager.CommandDeop(
-                new CommandModel(
-                    Frontend.FrontendManager,
-                    ChatModel,
-                    String.Join(" ", nicks.ToArray())
-                )
-            );
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    _IrcProtocolManager.CommandDeop(
+                        new CommandModel(
+                            Frontend.FrontendManager,
+                            ChatModel,
+                            String.Join(" ", nicks.ToArray())
+                        )
+                    );
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                }
+            });
         }
         
         private void _OnUserListMenuVoiceActivated(object sender, EventArgs e)
@@ -148,13 +162,19 @@ namespace Smuxi.Frontend.Gnome
             foreach (PersonModel person in persons) {
                 nicks.Add(person.ID);
             }
-            _IrcProtocolManager.CommandVoice(
-                new CommandModel(
-                    Frontend.FrontendManager,
-                    ChatModel,
-                    String.Join(" ", nicks.ToArray())
-                )
-            );
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    _IrcProtocolManager.CommandVoice(
+                        new CommandModel(
+                            Frontend.FrontendManager,
+                            ChatModel,
+                            String.Join(" ", nicks.ToArray())
+                        )
+                    );
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                }
+            });
         }
         
         private void _OnUserListMenuDevoiceActivated(object sender, EventArgs e)
@@ -171,13 +191,19 @@ namespace Smuxi.Frontend.Gnome
             foreach (PersonModel person in persons) {
                 nicks.Add(person.ID);
             }
-            _IrcProtocolManager.CommandDevoice(
-                new CommandModel(
-                    Frontend.FrontendManager,
-                    ChatModel,
-                    String.Join(" ", nicks.ToArray())
-                )
-            );
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    _IrcProtocolManager.CommandDevoice(
+                        new CommandModel(
+                            Frontend.FrontendManager,
+                            ChatModel,
+                            String.Join(" ", nicks.ToArray())
+                        )
+                    );
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                }
+            });
         }
         
         private void _OnUserListMenuKickActivated(object sender, EventArgs e)
@@ -190,13 +216,19 @@ namespace Smuxi.Frontend.Gnome
             }
 
             foreach (PersonModel person in persons) {
-                _IrcProtocolManager.CommandKick(
-                    new CommandModel(
-                        Frontend.FrontendManager,
-                        ChatModel,
-                        person.ID
-                    )
-                );
+                ThreadPool.QueueUserWorkItem(delegate {
+                    try {
+                        _IrcProtocolManager.CommandKick(
+                            new CommandModel(
+                                Frontend.FrontendManager,
+                                ChatModel,
+                                person.ID
+                            )
+                        );
+                    } catch (Exception ex) {
+                        Frontend.ShowException(ex);
+                    }
+                });
             }
         }
         
@@ -210,13 +242,19 @@ namespace Smuxi.Frontend.Gnome
             }
 
             foreach (PersonModel person in persons) {
-                _IrcProtocolManager.CommandKickban(
-                    new CommandModel(
-                        Frontend.FrontendManager,
-                        ChatModel,
-                        person.ID
-                    )
-                );
+                ThreadPool.QueueUserWorkItem(delegate {
+                    try {
+                        _IrcProtocolManager.CommandKickban(
+                            new CommandModel(
+                                Frontend.FrontendManager,
+                                ChatModel,
+                                person.ID
+                            )
+                        );
+                    } catch (Exception ex) {
+                        Frontend.ShowException(ex);
+                    }
+                });
             }
         }
         
@@ -234,13 +272,19 @@ namespace Smuxi.Frontend.Gnome
             foreach (PersonModel person in persons) {
                 nicks.Add(person.ID);
             }
-            _IrcProtocolManager.CommandBan(
-                new CommandModel(
-                    Frontend.FrontendManager,
-                    ChatModel,
-                    String.Join(" ", nicks.ToArray())
-                )
-            );
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    _IrcProtocolManager.CommandBan(
+                        new CommandModel(
+                            Frontend.FrontendManager,
+                            ChatModel,
+                            String.Join(" ", nicks.ToArray())
+                        )
+                    );
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                }
+            });
         }
         
         private void _OnUserListMenuUnbanActivated(object sender, EventArgs e)
@@ -256,13 +300,19 @@ namespace Smuxi.Frontend.Gnome
             foreach (PersonModel person in persons) {
                 nicks.Add(person.ID);
             }
-            _IrcProtocolManager.CommandUnban(
-                new CommandModel(
-                    Frontend.FrontendManager,
-                    ChatModel,
-                    String.Join(" ", nicks.ToArray())
-                )
-            );
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    _IrcProtocolManager.CommandUnban(
+                        new CommandModel(
+                            Frontend.FrontendManager,
+                            ChatModel,
+                            String.Join(" ", nicks.ToArray())
+                        )
+                    );
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                }
+            });
         }
         
         private void _OnUserListMenuQueryActivated(object sender, EventArgs e)
@@ -275,13 +325,19 @@ namespace Smuxi.Frontend.Gnome
             }
 
             foreach (PersonModel person in persons) {
-                _IrcProtocolManager.CommandMessageQuery(
-                    new CommandModel(
-                        Frontend.FrontendManager,
-                        ChatModel,
-                        person.ID
-                    )
-                );
+                ThreadPool.QueueUserWorkItem(delegate {
+                    try {
+                        _IrcProtocolManager.CommandMessageQuery(
+                            new CommandModel(
+                                Frontend.FrontendManager,
+                                ChatModel,
+                                person.ID
+                            )
+                        );
+                    } catch (Exception ex) {
+                        Frontend.ShowException(ex);
+                    }
+                });
             }
         }
 
@@ -295,35 +351,19 @@ namespace Smuxi.Frontend.Gnome
             }
 
             foreach (PersonModel person in persons) {
-                _IrcProtocolManager.CommandWhoIs(
-                    new CommandModel(
-                        Frontend.FrontendManager,
-                        ChatModel,
-                        person.ID
-                    )
-                );
-            }
-        }
-
-        protected override void OnPersonsRowActivated(object sender, Gtk.RowActivatedArgs e)
-        {
-            Trace.Call(sender, e);
-
-            base.OnPersonsRowActivated(sender, e);
-
-            IList<PersonModel> persons = GetSelectedPersons();
-            if (persons == null) {
-                return;
-            }
-
-            foreach (PersonModel person in persons) {
-                _IrcProtocolManager.CommandMessageQuery(
-                    new CommandModel(
-                        Frontend.FrontendManager,
-                        ChatModel,
-                        person.ID
-                    )
-                );
+                ThreadPool.QueueUserWorkItem(delegate {
+                    try {
+                        _IrcProtocolManager.CommandWhoIs(
+                            new CommandModel(
+                                Frontend.FrontendManager,
+                                ChatModel,
+                                person.ID
+                            )
+                        );
+                    } catch (Exception ex) {
+                        Frontend.ShowException(ex);
+                    }
+                });
             }
         }
 
@@ -396,38 +436,8 @@ namespace Smuxi.Frontend.Gnome
             );
             invite_to_item.Submenu = invite_to_menu_item;
             PersonMenu.Append(invite_to_item);
-        }
 
-        protected override int SortPersonListStore(Gtk.TreeModel model,
-                                                   Gtk.TreeIter iter1,
-                                                   Gtk.TreeIter iter2)
-        {
-            Gtk.ListStore liststore = (Gtk.ListStore) model;
-            
-            IrcGroupPersonModel person1 = (IrcGroupPersonModel) liststore.GetValue(iter1, 0);
-            IrcGroupPersonModel person2 = (IrcGroupPersonModel) liststore.GetValue(iter2, 0);
-            
-            int status1 = 0;
-            if (person1.IsOp) {
-                status1 += 2;
-            } else if (person1.IsVoice) {
-                status1 += 1;
-            }
-            
-            int status2 = 0;
-            if (person2.IsOp) {
-                status2 += 2;
-            } else if (person2.IsVoice) {
-                status2 += 1;
-            }
-            
-            int res = status2.CompareTo(status1);
-            if (res == 0 ) {
-                // the mode is equal, so the name decides
-                return base.SortPersonListStore(model, iter1, iter2);
-            }
-            
-            return res;
+            PersonMenu.ShowAll();
         }
 
         private static string _(string msg)
