@@ -51,6 +51,18 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        public Gtk.Entry NameEntry {
+            get {
+                return f_NameEntry;
+            }
+        }
+
+        public Gtk.Button FindButton {
+            get {
+                return f_FindButton;
+            }
+        }
+
         public FindGroupChatDialog(Gtk.Window parent, IProtocolManager protocolManager) :
                               base(null, parent, Gtk.DialogFlags.DestroyWithParent)
         {
@@ -93,7 +105,8 @@ namespace Smuxi.Frontend.Gnome
             
             try {
                 string nameFilter = f_NameEntry.Text.Trim();
-                if (String.IsNullOrEmpty(nameFilter)) {
+                if (!(Frontend.EngineVersion >= new Version("0.8.1")) &&
+                    String.IsNullOrEmpty(nameFilter)) {
                     Gtk.MessageDialog md = new Gtk.MessageDialog(
                         this,
                         Gtk.DialogFlags.Modal,
@@ -150,6 +163,7 @@ namespace Smuxi.Frontend.Gnome
                         });
                     }
                 }));
+                f_FindThread.IsBackground = true;
                 f_FindThread.Start();
             } catch (Exception ex) {
                 Frontend.ShowException(ex);
