@@ -386,17 +386,22 @@ namespace Smuxi.Frontend.Gnome
                 // feature introduced in >= 0.8.1
                 Gtk.ComboBox persistencyTypeComboBox =
                     ((Gtk.ComboBox)_Glade["PersistencyTypeComboBox"]);
-                var persistencyType = (MessageBufferPersistencyType) Enum.Parse(
-                    typeof(MessageBufferPersistencyType),
-                    (string) Frontend.UserConfig["MessageBuffer/PersistencyType"]
-                );
-                i = 0;
-                foreach (object[] row in (Gtk.ListStore) persistencyTypeComboBox.Model) {
-                    if (((MessageBufferPersistencyType) row[0]) == persistencyType) {
-                        persistencyTypeComboBox.Active = i;
-                        break;
+                try {
+                    var persistencyType = (MessageBufferPersistencyType) Enum.Parse(
+                        typeof(MessageBufferPersistencyType),
+                        (string) Frontend.UserConfig["MessageBuffer/PersistencyType"]
+                    );
+                    i = 0;
+                    foreach (object[] row in (Gtk.ListStore) persistencyTypeComboBox.Model) {
+                        if (((MessageBufferPersistencyType) row[0]) == persistencyType) {
+                            persistencyTypeComboBox.Active = i;
+                            break;
+                        }
+                        i++;
                     }
-                    i++;
+                } catch (ArgumentException) {
+                    // for forward compatibility with newer engines
+                    persistencyTypeComboBox.Active = -1;
                 }
                 ((Gtk.SpinButton)_Glade["VolatileMaxCapacitySpinButton"]).Value =
                     (double)(int)Frontend.UserConfig["MessageBuffer/Volatile/MaxCapacity"];
@@ -437,7 +442,15 @@ namespace Smuxi.Frontend.Gnome
                     ((Gtk.RadioButton)_Glade["TabPositionRadioButtonNone"]).Active = true;
                 break;
             }
-            
+            if (Frontend.UserConfig["Interface/Notebook/AutoSwitchPersonChats"] != null) {
+                ((Gtk.CheckButton) _Glade["AutoSwitchPersonChatsCheckButton"]).Active =
+                    (bool) Frontend.UserConfig["Interface/Notebook/AutoSwitchPersonChats"];
+            }
+            if (Frontend.UserConfig["Interface/Notebook/AutoSwitchGroupChats"] != null) {
+                ((Gtk.CheckButton) _Glade["AutoSwitchGroupChatsCheckButton"]).Active =
+                    (bool) Frontend.UserConfig["Interface/Notebook/AutoSwitchGroupChats"];
+            }
+
             // Interface/Notebook/Channel
             switch ((string)Frontend.UserConfig["Interface/Notebook/Channel/UserListPosition"]) {
                 case "left":
@@ -682,17 +695,20 @@ namespace Smuxi.Frontend.Gnome
             // MessageBuffer
             if (Frontend.EngineVersion >= new Version("0.8.1")) {
                 var persistencyTypeComboBox = (Gtk.ComboBox) _Glade["PersistencyTypeComboBox"];
-                var persistencyType = MessageBufferPersistencyType.Volatile;
-                i = 0;
-                foreach (object[] row in (Gtk.ListStore) persistencyTypeComboBox.Model) {
-                    if (persistencyTypeComboBox.Active == i) {
-                        persistencyType = (MessageBufferPersistencyType) row[0];
-                        break;
+                // for forward compatibility with newer engines
+                if (persistencyTypeComboBox.Active != -1) {
+                    var persistencyType = MessageBufferPersistencyType.Volatile;
+                    i = 0;
+                    foreach (object[] row in (Gtk.ListStore) persistencyTypeComboBox.Model) {
+                        if (persistencyTypeComboBox.Active == i) {
+                            persistencyType = (MessageBufferPersistencyType) row[0];
+                            break;
+                        }
+                        i++;
                     }
-                    i++;
+                    Frontend.UserConfig["MessageBuffer/PersistencyType"] =
+                        persistencyType.ToString();
                 }
-                Frontend.UserConfig["MessageBuffer/PersistencyType"] =
-                    persistencyType.ToString();
                 Frontend.UserConfig["MessageBuffer/Volatile/MaxCapacity"] =
                     (int)((Gtk.SpinButton)_Glade["VolatileMaxCapacitySpinButton"]).Value;
                 Frontend.UserConfig["MessageBuffer/Persistent/MaxCapacity"] =
@@ -703,6 +719,7 @@ namespace Smuxi.Frontend.Gnome
             Frontend.UserConfig["Interface/ShowAdvancedSettings"] =
                 ((Gtk.CheckButton)_Glade["ShowAdvancedSettingsCheckButton"]).Active;
 
+            // Interface/Notebook
             Frontend.UserConfig["Interface/Notebook/TimestampFormat"] =
                 ((Gtk.Entry)_Glade["TimestampFormatEntry"]).Text;
             Frontend.UserConfig["Interface/Notebook/BufferLines"] =
@@ -727,7 +744,13 @@ namespace Smuxi.Frontend.Gnome
                 tab_position = "none";
             }
             Frontend.UserConfig["Interface/Notebook/TabPosition"] = tab_position;
-            
+
+            Frontend.UserConfig["Interface/Notebook/AutoSwitchPersonChats"] =
+                ((Gtk.CheckButton)_Glade["AutoSwitchPersonChatsCheckButton"]).Active;
+            Frontend.UserConfig["Interface/Notebook/AutoSwitchGroupChats"] =
+                ((Gtk.CheckButton)_Glade["AutoSwitchGroupChatsCheckButton"]).Active;
+
+            // Interface/Notebook/Channel
             string userlist_position = null;
              if (((Gtk.RadioButton)_Glade["UserListPositionRadioButtonLeft"]).Active) {
                 userlist_position = "left";
