@@ -21,7 +21,6 @@
  */
 
 using System;
-using System.Drawing;
 using System.Threading;
 using System.Collections.Generic;
 using Smuxi.Common;
@@ -509,7 +508,7 @@ namespace Smuxi.Frontend.Gnome
                 msg.AppendMessage(_("Low Bandwidth Mode is active: no messages synced."));
                 AddMessage(msg.ToMessage());
             } else {
-                if (SyncedMessages.Count > 0) {
+                if (SyncedMessages != null) {
                     // TODO: push messages in batches and give back control to
                     // GTK+ in between for blocking the GUI thread less
                     foreach (MessageModel msg in SyncedMessages) {
@@ -565,7 +564,7 @@ namespace Smuxi.Frontend.Gnome
 
             ThreadPool.QueueUserWorkItem(delegate {
                 try {
-                    ChatModel.ProtocolManager.CloseChat(
+                    ProtocolManager.CloseChat(
                         Frontend.FrontendManager,
                         ChatModel
                     );
